unit Main;

interface

uses         
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls, Buttons, VBScript_RegExp_55_TLB;

type
    TThreadCreator = class(TThread)
    protected
        FMaxCount: Integer;
        FCurCount: Integer;
        FRetry302: Boolean;
        procedure CheckConfig();
    protected
        FMessage: String;
        FError: Boolean;
        procedure PutLog();
    protected
        FID: Integer;
        FAddResult: Boolean;
        procedure AddThread();
    protected
        procedure Execute(); override;
    end;

    TPageInfo = record
        KBID: String;
        URL: String;
        RESCODE: Integer;
        TITLE: String;
        DESCRIPTION: String;
        KEYWORDS: String;
        KBPARENTS: String;
        GETBEGIN: String;
        GETEND: String;
        Status: String;
        Charset: String;
    end;

    TDownloadThread = class(TThread)
    protected
        procedure AppendSelf();
        procedure RemoveSelf();
    protected
        FStream: TStringStream;
        FRegBuf: TRegExp;
        FPageInfo: TPageInfo;
        function CallbackDownload(const Buf; const BufLen: Integer): Boolean;
        procedure Execute(); override;
    public
        constructor Create(const AID, AURL: String);
        destructor Destroy(); override;
    end;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    MemLog: TMemo;
    AfterShowTrigger: TPaintBox;
    BtnVersion: TBitBtn;
    Panel2: TPanel;
    LboxThreads: TListBox;
    Label1: TLabel;
    EditThreadCount: TEdit;
    UpdnThreadCount: TUpDown;
    Panel3: TPanel;
    Label2: TLabel;
    EditWord: TEdit;
    BtnSearch: TBitBtn;
    ChkRetry302: TCheckBox;
    Label3: TLabel;
    MemErr: TMemo;
    Splitter1: TSplitter;
    MemResult: TMemo;
    BtnStatus: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AfterShowTriggerPaint(Sender: TObject);
    procedure BtnVersionClick(Sender: TObject);
    procedure BtnSearchClick(Sender: TObject);
    procedure BtnStatusClick(Sender: TObject);
  private
    procedure PutLog(const AMsg: String; const AErr: Boolean);
    procedure OnFatalError();
  private
    FThreadCreator: TThreadCreator;
    FStrBuf: TStringList;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

uses
    Info, DatabaseIO, Jconvert, Http, StrUtils;

const
    MSKBINDEX: String = 'MSKBINDEX';    //  e[u
    MSKBINDEXV: String = 'MSKBINDEXV';   //  r[
    MIN_KBID: Integer =   99999;        //  ŏԍ - 1
    MAX_KBID: Integer = 1000000;        //  ő啶ԍ + 1
    MSKB_URL: String = 'http://support.microsoft.com/default.aspx?scid=kb;ja;';

{$R *.dfm}

//=============================================================================
//  Utility
//=============================================================================

//-----------------------------------------------------------------------------
//  Tu}b`1ڂ߂
function GetSubMatchItem0(ARegBuf: TRegExp; const AData, APattern: String): String;
var
    LMatches, LSubMatches: OleVariant;
begin
    Result := '';
    ARegBuf.Pattern := APattern;
    ARegBuf.Global := False;            //  ŏ
    ARegBuf.IgnoreCase := True;         //  啶E
    LMatches := ARegBuf.Execute(AData); //  
    if (LMatches.Count > 0) then
    begin
        LSubMatches := LMatches.Item[0].SubMatches;
        if (LSubMatches.Count > 0) then
        begin
            Result := LSubMatches.Item[0];
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  R[hϊ
procedure ConvertCoding(const ACharset: String; var AData: String);
begin
    //  Shift_JIS
    if (AnsiContainsText(ACharset, 'Shift_JIS')) then
    begin
        //  ̂܂
    end

    //  xsjis
    else if (AnsiContainsText(ACharset, 'xsjis')) then
    begin
        //  ̂܂
    end

    //  UTF-8
    else if (AnsiContainsText(ACharset, 'UTF-8')) then
    begin
        AData := Utf8ToAnsi(AData);
    end

    //  EUC̏ꍇ
    else if (AnsiContainsText(ACharset, 'euc')) then
    begin
        AData := euc2sjis(AData);
    end

    //  s̏ꍇ
    else
    begin
        AData := ConvertJCode(AData, SJIS_OUT);
    end;
end;

//-----------------------------------------------------------------------------
//  ESCL[Ń[vf
//  http://forum.nifty.com/fdelphi/samples/00782.html
function IsKeyDown(const K: Word): Boolean;
begin
    Result := (GetAsyncKeyState(K) <> 0);
end;

//=============================================================================
//  TThreadCreator
//=============================================================================

//-----------------------------------------------------------------------------
//  tH[󋵂ǂݎ
procedure TThreadCreator.CheckConfig();
begin
    //  őXbh
    FMaxCount := MainForm.UpdnThreadCount.Position;

    //  ݍ쐬ς݂̃Xbh
    FCurCount := MainForm.LboxThreads.Items.Count;

    //  O񑶍݂Ȃy[WgC邩
    FRetry302 := MainForm.ChkRetry302.Checked;
end;

//-----------------------------------------------------------------------------
//  tH[ɃG[߂
procedure TThreadCreator.PutLog();
begin
    MainForm.PutLog(FMessage, FError);
end;

//-----------------------------------------------------------------------------
//  _E[hXbh쐬
procedure TThreadCreator.AddThread();
var
    LID: String;
    LURL: String;
    LQuery: String;
    LQueries: TStringList;
    LThread: TThread;
begin
    LQueries := TStringList.Create();

    LID := IntToStr(FID);
    LURL := MSKB_URL + LID;

    LQuery := 'DELETE FROM MSKBINDEX WHERE (KBID=%s)';
    LQuery := Format(LQuery, [Quote(LID)]);
    LQueries.Add(LQuery);

    LQuery := 'INSERT INTO MSKBINDEX(KBID,URL,RESCODE) VALUES(%s,%s,0)';
    LQuery := Format(LQuery, [Quote(LID), Quote(LURL)]);
    LQueries.Add(LQuery);

    CS.Enter();
    try
        FAddResult := DB.Execute(LQueries, FMessage);
    finally
        CS.Leave();
    end;

    //  _E[hXbh쐬
    if (FAddResult) then
    begin
        LThread := TDownloadThread.Create(LID, LURL);
        LThread.Resume();
    end;

    LQueries.Free();
end;

//-----------------------------------------------------------------------------
//  {
procedure TThreadCreator.Execute();
var
    LQuery: String;
    LSet: OleVariant;
begin
    inherited;

    while not Terminated do
    begin
        //  󋵃`FbN
        Synchronize(CheckConfig);

        //  őXbhɓB܂Ń_E[hXbh쐬
        if (FCurCount < FMaxCount) then
        begin
            //-----------------------------------------------------------------
            //  _E[h镶ԍ肷
            FID := -1;

            //  O_E[h悤ƂĂłĂȂ̂𒊏o
            if (FID < 0) then
            begin
                LQuery := 'SELECT MIN(KBID) AS KBID FROM MSKBINDEX WHERE (RESCODE IS NULL)';
                CS.Enter();
                try
                    if (DB.OpenQuery(LQuery, LSet, FMessage)) then
                    begin
                        FID := NVL(DB.FieldValue(LSet, 'KBID'), FID);
                        DB.CloseQuery(LSet);
                    end
                    else
                    begin
                        FError := True;
                        Synchronize(PutLog);
                    end;
                finally              
                    CS.Leave();
                end;
            end;

            //  O_E[hɃy[W݂Ȃ̂gC
            if (FID < 0) and (FRetry302) then
            begin
                LQuery := 'SELECT MIN(KBID) AS KBID FROM MSKBINDEX WHERE (RESCODE=302)';
                CS.Enter();
                try
                    if (DB.OpenQuery(LQuery, LSet, FMessage)) then
                    begin
                        FID := NVL(DB.FieldValue(LSet, 'KBID'), FID);
                        DB.CloseQuery(LSet);
                    end
                    else
                    begin
                        FError := True;
                        Synchronize(PutLog);
                    end;
                finally
                    CS.Leave();
                end;
            end;

            //  f[^ȂΐVԍ߂
            if (FID < 0) then
            begin
                //  _E[hς݂̍őԍ߂
                LQuery := 'SELECT MAX(KBID) AS KBID FROM MSKBINDEX';
                CS.Enter();
                try
                    if (DB.OpenQuery(LQuery, LSet, FMessage)) then
                    begin
                        //  ߂Ă̏ꍇ 100000 
                        FID := NVL(DB.FieldValue(LSet, 'KBID'), MIN_KBID) + 1;
                        DB.CloseQuery(LSet);
                    end
                    else
                    begin
                        FError := True;
                        Synchronize(PutLog);
                    end;
                finally
                    CS.Leave();
                end;
            end;

            //  _E[hΏۂ܂΃_E[hJn
            if ((MIN_KBID < FID) and (FID < MAX_KBID)) then
            begin
                Synchronize(AddThread);

                if (not FAddResult) then
                begin
                    FError := True;
                    Synchronize(PutLog);
                end;
            end;
        end;

        Sleep(1);
    end;
end;

//=============================================================================
//  TDownloadThread
//=============================================================================

//-----------------------------------------------------------------------------
//  tH[Ɏo^
procedure TDownloadThread.AppendSelf();
begin
    MainForm.LboxThreads.Items.AddObject(FPageInfo.URL, Self);
end;

//-----------------------------------------------------------------------------
//  tH[玩
procedure TDownloadThread.RemoveSelf();
begin
    MainForm.LboxThreads.Items.Delete(MainForm.LboxThreads.Items.IndexOfObject(Self));
end;

//-----------------------------------------------------------------------------
//  RXgN^
constructor TDownloadThread.Create(const AID, AURL: String);
begin
    inherited Create(True);

    //  \̃Zbg
    FPageInfo.RESCODE := 0;
    FPageInfo.KBID := AID;
    FPageInfo.URL := AURL;

    //  tH[ɓo^
    Synchronize(AppendSelf);

    //  _E[hɎgpNXCX^X쐬
    FStream := TStringStream.Create('');
    FRegBuf := TRegExp.Create(nil);

    //  [h
    FreeOnTerminate := True;
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TDownloadThread.Destroy();
begin
    FStream.Free();
    FRegBuf.Free();

    //  tH[
    Synchronize(RemoveSelf);

    inherited;
end;

//-----------------------------------------------------------------------------
//  _E[hpR[obN֐
function TDownloadThread.CallbackDownload(const Buf; const BufLen: Integer): Boolean;
var
    LCancelCondition: Boolean;
begin
    FStream.Write(Buf, BufLen);

    //  "<XMLHEADER" ŃLZ(ȏ͕sv)
    LCancelCondition := AnsiContainsText(FStream.DataString, '<XMLHEADER');
    Result := not LCancelCondition;
end;

//-----------------------------------------------------------------------------
//  {
procedure TDownloadThread.Execute();
var
    LHost, LPage, LCommand: String;
    LData: String;
    LPattern: String;
    LQuery: String;
    LErr: String;
begin
    //  擾Jn
    FPageInfo.GETBEGIN := FormatDateTime('yyyymmddhhnnss', Now());

    //  URL
    if (SplitUrl(FPageInfo.URL, LHost, LPage)) then
    begin
        //  R}h쐬
        LCommand := Format(
            'GET %s HTTP/1.0'#13#10
          + 'HOST: %s:80'#13#10
          + 'User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)'
          + #13#10#13#10, [LPage, LHost]);

        //  _E[h
        if (DownloadViaHTTP(LHost, LCommand, CallbackDownload)) then
        begin
            //  wb_̃Xg[f[^ڂ
            LData := FStream.DataString;
            FStream.Create('');

            //  Xe[^Xo
            LPattern := '(^HTTP/[0-9]\.[0-9][\s]+[0-9]+[\s]+[^\s]*)';
            FPageInfo.Status := GetSubMatchItem0(FRegBuf, LData, LPattern);

            //  Xe[^XR[ho
            LPattern := '^HTTP/[0-9]\.[0-9][\s]+([0-9]+)[\s]+[^\s]*';
            FPageInfo.RESCODE := StrToIntDef(GetSubMatchItem0(FRegBuf, FPageInfo.Status, LPattern), 0);

            //  `FbN
            if (FPageInfo.RESCODE = 200) then
            begin
                //  charseto(T[o[X|XMETA^Oɔ)
                LPattern := ';[\s]*charset[\s]*=[\s]*([^\s">]*)';
                FPageInfo.Charset := Trim(GetSubMatchItem0(FRegBuf, LData, LPattern));

                //  R[hϊ
                ConvertCoding(FPageInfo.Charset, LData);

                //  sƂE^uƂ
                LData := StringReplace(LData, #13, '', [rfReplaceAll]);
                LData := StringReplace(LData, #10, '', [rfReplaceAll]);
                LData := StringReplace(LData, #9, ' ', [rfReplaceAll]);

                //  TITLE ̔
                LPattern := '<title[^>]*>([^<]*)</title>';
                FPageInfo.TITLE := GetSubMatchItem0(FRegBuf, LData, LPattern);

                //  DESCRIPTION ̔-Xebv1
                LPattern := '<meta[\s]*name[\s]*=[\s]*"?description"?[\s]*([^>]*)>';
                FPageInfo.DESCRIPTION := Trim(GetSubMatchItem0(FRegBuf, LData, LPattern));

                //  DESCRIPTION ̔-Xebv2
                LPattern := 'content[\s]*="?([^">]*)"?';
                FPageInfo.DESCRIPTION := GetSubMatchItem0(FRegBuf, FPageInfo.DESCRIPTION, LPattern);

                //  KEYWORDS ̔-Xebv1
                LPattern := '<meta[\s]*name[\s]*=[\s]*"?keywords"?[\s]*([^>]*)>';
                FPageInfo.KEYWORDS := Trim(GetSubMatchItem0(FRegBuf, LData, LPattern));

                //  KEYWORDS ̔-Xebv2
                LPattern := 'content[\s]*="?([^">]*)"?';
                FPageInfo.KEYWORDS := GetSubMatchItem0(FRegBuf, FPageInfo.KEYWORDS, LPattern);

                //  KBPARENTS ̔-Xebv1
                LPattern := '<meta[\s]*name[\s]*=[\s]*"?kbparents"?[\s]*([^>]*)>';
                FPageInfo.KBPARENTS := Trim(GetSubMatchItem0(FRegBuf, LData, LPattern));

                //  KBPARENTS ̔-Xebv2
                LPattern := 'content[\s]*="?([^">]*)"?';
                FPageInfo.KBPARENTS := GetSubMatchItem0(FRegBuf, FPageInfo.KBPARENTS, LPattern);
            end;
        end;
    end
    else
    begin
        //
    end;

    //  擾
    FPageInfo.GETEND := FormatDateTime('yyyymmddhhnnss', Now());

    //  ۑ
    LQuery := 'UPDATE MSKBINDEX SET '
                             + 'RESCODE=%d,'
                             + 'TITLE=%s,'
                             + 'DESCRIPTION=%s,'
                             + 'KEYWORDS=%s,'
                             + 'KBPARENTS=%s,'
                             + 'GETBEGIN=%s,'
                             + 'GETEND=%s'
            + ' WHERE KBID=%s';
    LQuery := Format(LQuery, [FPageInfo.RESCODE,
                              Quote(FPageInfo.TITLE),
                              Quote(FPageInfo.DESCRIPTION),
                              Quote(FPageInfo.KEYWORDS),
                              Quote(FPageInfo.KBPARENTS),
                              Quote(FPageInfo.GETBEGIN),
                              Quote(FPageInfo.GETEND),
                              Quote(FPageInfo.KBID)]);

    //  sĂʒm悤Ȃ(ז)
    CS.Enter();
    try
        DB.Execute(LQuery, LErr);
    finally
        CS.Leave();
    end;
end;

//=============================================================================
//  TMainForm
//=============================================================================

//-----------------------------------------------------------------------------
//  Oo
procedure TMainForm.PutLog(const AMsg: String; const AErr: Boolean);
const
    LOG_TYPE: array[Boolean] of String = ('', '');
begin
    MemLog.Lines.Add(LOG_TYPE[AErr] + FormatDateTime('yyyy/mm/dd hh:nn:ss ', Now()) + AMsg);
end;

//-----------------------------------------------------------------------------
//  vIG[
procedure TMainForm.OnFatalError();
begin
    AppMessageBox('SORRY, FATAL ERROR OCCURED.', MB_OK or MB_ICONERROR);
end;

//-----------------------------------------------------------------------------
//  tH[쐬
procedure TMainForm.FormCreate(Sender: TObject);
var
    LErr: String;
begin
    SetFormCaption(Self);

    DB := TDatabaseJet.Create();
    if (not DB.Open('', '', ChangeFileExt(ParamStr(0), '.mdb'), LErr)) then
    begin
        PutLog(LErr, True);
        OnFatalError();
    end
    else
    begin
        PutLog('CONNECTED:' + DB.DatabaseName, False);
    end;

    //  
    FThreadCreator := nil;
    FStrBuf := TStringList.Create();
end;

//-----------------------------------------------------------------------------
//  tH[j
procedure TMainForm.FormDestroy(Sender: TObject);
var
    LErr: String;
begin
    if (Assigned(FThreadCreator)) then
    begin
        FThreadCreator.Terminate();
        FThreadCreator.Free();
        FThreadCreator := nil;
    end;

    //  _E[hJnĂ܂ĂXbh͏I点
    while (LboxThreads.Items.Count > 0) do
    begin
        Application.ProcessMessages();
    end;

    FStrBuf.Free();

    //  œKďI
    TDatabaseJet(DB).CompactDatabase(LErr);
    DB.Close();
    DB.Free();
end;

//-----------------------------------------------------------------------------
//  tH[\ɏ
procedure TMainForm.AfterShowTriggerPaint(Sender: TObject);
var
    LBox: TPaintBox;
    LBuf: TStringList;
    LIndex: Integer;
    LQuery: String;
    LErr: String;
    LChanged: Boolean;
begin
    //  \ɑ`揈gāuOnAfterShowvCxg܂˂
    LBox := TPaintBox(Sender);
    LBox.OnPaint := nil;
    LBox.Hide();

    //  f[^x[Xt@C`FbN
    LBuf := TStringList.Create();
    DB.GetTableNames(LBuf);
    LBuf.Sorted := True;

    LChanged := False;

    //  ݂Ȃ!
    if (not LBuf.Find(MSKBINDEX, LIndex)) then
    begin
        //  쐬
        LQuery := 'CREATE TABLE MSKBINDEX ('
                 + ' KBID TEXT(6) NOT NULL,'
                 + ' URL TEXT(100) NOT NULL,'
                 + ' RESCODE INTEGER,'
                 + ' TITLE MEMO,'
                 + ' DESCRIPTION MEMO,'
                 + ' KEYWORDS MEMO,'
                 + ' KBPARENTS MEMO,'
                 + ' GETBEGIN TEXT(14),'
                 + ' GETEND TEXT(14),'
                 + ' CONSTRAINT PK_MSKBINDEX PRIMARY KEY (KBID)'
                 + ');';

        if (not DB.Execute(LQuery, LErr)) then
        begin
            PutLog(LErr, True);
            OnFatalError();
            Exit;
        end
        else
        begin
            PutLog('TABLE CREATED:' + MSKBINDEX, False);
        end;

        //  ύXL
        LChanged := True;
    end;

    //  ݂Ȃ!
    if (not LBuf.Find(MSKBINDEXV, LIndex)) then
    begin
        //  ЂƂ쐬
        LQuery := 'CREATE VIEW MSKBINDEXV AS'
                + ' SELECT KBID,URL,TITLE,DESCRIPTION,KEYWORDS,KBPARENTS'
                + ' FROM MSKBINDEX WHERE (RESCODE=200)';

        if (not DB.Execute(LQuery, LErr)) then
        begin
            PutLog(LErr, True);
            OnFatalError();
            Exit;
        end
        else
        begin
            PutLog('VIEW CREATED:' + MSKBINDEXV, False);
        end;

        //  ύXL
        LChanged := True;
    end;

    //  JȂ
    if (LChanged) then
    begin
        //  JȂ(ȂƐVKɒǉe[uȂ)
        if (not DB.ReOpen(LErr)) then
        begin
            PutLog(LErr, True);
            OnFatalError();
            Exit;
        end;
    end;

    //  ̒(̂邢قɓ̂ŖȂ)
    if (not TDatabaseJet(DB).SyncNullableAllowZeroLength(MSKBINDEX, LErr)) then
    begin
        PutLog(LErr, True);
        OnFatalError();
        Exit;
    end
    else
    begin
        PutLog('SYNC NULLABLE AND ALLOWZEROLENGTH:' + MSKBINDEX, False);
    end;

    //  ݃f[^|
    LQuery := 'UPDATE MSKBINDEX SET RESCODE=NULL WHERE (RESCODE=0)';
    if (not DB.Execute(LQuery, LErr)) then
    begin
        PutLog(LErr, True);
        OnFatalError();
        Exit;
    end
    else
    begin
        PutLog('DELETE JUNK RECORDS:' + LQuery, False);
    end;

    //  Xbh쐬pXbhN
    if (not Assigned(FThreadCreator)) then
    begin
        FThreadCreator := TThreadCreator.Create(True);
        //FThreadCreator.Priority := tpNormal;
        FThreadCreator.Resume();
        PutLog('THREAD CREATOR CREATED', False);
    end;
end;

//-----------------------------------------------------------------------------
//  o[W\
procedure TMainForm.BtnVersionClick(Sender: TObject);
begin
    OpenInfoForm();
end;

//-----------------------------------------------------------------------------
//  
procedure TMainForm.BtnSearchClick(Sender: TObject);
var
    LWords: String;
    LBuf: TStringList;
    LQuery: String;
    i: Integer;
    LWhere: String;
    LSet: OleVariant;
    LErr: String;
    LResult: String;
    LCount: Integer;
begin
    Screen.Cursor := crHourGlass;

    //  dL[[h폜
    LBuf := TStringList.Create();
    LBuf.Sorted := True;
    LBuf.Duplicates := dupIgnore;

    LWords := EditWord.Text;
    LWords := StringReplace(LWords, '@', ' ', [rfReplaceAll]);
    LWords := StringReplace(LWords, ',', #1, [rfReplaceAll]);
    FStrBuf.CommaText := LWords;
    while (FStrBuf.Count > 0) do
    begin
        LWords := Trim(FStrBuf.Strings[0]);
        if (LWords <> '') then
        begin
            LWords := StringReplace(LWords, #1, ',', [rfReplaceAll]);
            LBuf.Add(LWords);
        end;
        FStrBuf.Delete(0);
    end;

    for i := 0 to LBuf.Count - 1 do
    begin
        FStrBuf.Add('LEFT(KBID&URL&TITLE&DESCRIPTION&KEYWORDS,5000) LIKE ' + Quote(LBuf.Strings[i]));
    end;

    if (FStrBuf.Count > 0) then
    begin
        LWhere := ' WHERE (' + FStrBuf.Strings[0] + ')';
    end;
    for i := 1 to FStrBuf.Count - 1 do
    begin
        LWhere := LWhere + ' AND (' + FStrBuf.Strings[1] + ')';
    end;

    //  LIKÊ߂̏
    LQuery := 'SELECT KBID,URL,TITLE,DESCRIPTION,KEYWORDS FROM MSKBINDEXV' + LWhere;

    FStrBuf.Clear();
    LBuf.Free();

    //  NG[s
    CS.Enter();
    MemResult.Lines.BeginUpdate();
    try
        if (DB.OpenQuery(LQuery, LSet, LErr)) then
        begin
            MemErr.Text := LQuery;
            MemErr.Lines.Add('ESCL[Œfł܂B');

            MemResult.Clear();
            LCount := 0;
            while (not DB.IsXOF(LSet)) do
            begin
                Inc(LCount);

                for i := 0 to DB.FieldCount(LSet) - 1 do
                begin
                    LResult := NVL(DB.FieldValue(LSet, i), '');
                    if (LResult <> '') then
                    begin
                        MemResult.Lines.Add(LResult);
                    end;
                end;
                MemResult.Lines.Add('');

                DB.MoveNext(LSet);

                //  ESCŒf
                if (IsKeyDown(VK_ESCAPE)) then
                begin
                    MemResult.Lines.Insert(0, '');
                    MemResult.Lines.Insert(0, 'f܂');
                    Break;
                end;
            end;

            //  Ĵ͕
            DB.CloseQuery(LSet);

            MemResult.Lines.Insert(0, StringOfChar('-', 50));
            MemResult.Lines.Insert(0, Format('[%d ] qbg܂B', [LCount]));
            MemResult.Lines.Insert(0, StringOfChar('-', 50));
        end
        else
        begin
            MemErr.Text := LErr;
        end;
    finally
        MemResult.Lines.EndUpdate();
        CS.Leave();
    end;

    Screen.Cursor := crDefault;
end;

//-----------------------------------------------------------------------------
//  Xe[^X
procedure TMainForm.BtnStatusClick(Sender: TObject);
var
    LQuery: String;
    LSet: OleVariant;

    LCount: Integer;
    LCount200: Integer;
    LCount302: Integer;
    LMinID: String;
    LMaxID: String;

    LCountErr: String;
    LCount200Err: String;
    LCount302Err: String;
    LMinIDErr: String;
    LMaxIDErr: String;
begin
    LCount := -1;
    LCount200 := -1;
    LCount302 := -1;
    LMinID := '';
    LMaxID := '';

    CS.Enter();
    try
        //  S
        LQuery := 'SELECT COUNT(*) FROM MSKBINDEX';
        if (DB.OpenQuery(LQuery, LSet, LCountErr)) then
        begin
            LCount := DB.FieldValue(LSet, 0);
            DB.CloseQuery(LSet);
        end;

        //  ݌
        LQuery := 'SELECT COUNT(*) FROM MSKBINDEX WHERE (RESCODE=200)';
        if (DB.OpenQuery(LQuery, LSet, LCount200Err)) then
        begin
            LCount200 := DB.FieldValue(LSet, 0);
            DB.CloseQuery(LSet);
        end;

        //  s݌
        LQuery := 'SELECT COUNT(*) FROM MSKBINDEX WHERE (RESCODE=302)';
        if (DB.OpenQuery(LQuery, LSet, LCount302Err)) then
        begin
            LCount302 := DB.FieldValue(LSet, 0);
            DB.CloseQuery(LSet);
        end;

        //  ŏԍ
        LQuery := 'SELECT MIN(KBID) FROM MSKBINDEX WHERE (RESCODE=200)';
        if (DB.OpenQuery(LQuery, LSet, LMinIDErr)) then
        begin
            LMinID := nvl(DB.FieldValue(LSet, 0), '');
            DB.CloseQuery(LSet);
        end;

        //  őԍ
        LQuery := 'SELECT MAX(KBID) FROM MSKBINDEX WHERE (RESCODE=200)';
        if (DB.OpenQuery(LQuery, LSet, LMaxIDErr)) then
        begin
            LMaxID := nvl(DB.FieldValue(LSet, 0), '');
            DB.CloseQuery(LSet);
        end;
    finally
        CS.Leave();
    end;

    FStrBuf.Add('|||@_E[h󋵁@|||');
    FStrBuf.Add('');
    FStrBuf.Add(Format('@@@[Š]@%d @%s', [LCount, LCountErr]));
    FStrBuf.Add(Format('@@@[݌]@%d @%s', [LCount200, LCount200Err]));
    FStrBuf.Add(Format('@@@[s݌]@%d @%s', [LCount302, LCount302Err]));
    FStrBuf.Add(Format('@@@[ŏԍ]@%s@%s', [LMinID, LMinIDErr]));
    FStrBuf.Add(Format('@@@[őԍ]@%s@%s', [LMaxID, LMaxIDErr]));

    AppMessageBox(FStrBuf.Text, MB_OK or MB_ICONINFORMATION);
end;

end.
